<?php
namespace Financeiro\Logic\ContasReceber\Boleto\EmissaoBoleto;

use \Financeiro\Logic\ContasReceber\Boleto\EmissaoBoleto\Exception\ExcecaoBoletoGeral;
use Financeiro\Logic\ContasReceber\Boleto\EmissaoBoleto\Exception\CodigoCompensacaoInvalido;

/**
 * Created by JetBrains PhpStorm.
 * User: alanoliveira
 * Date: 11/04/13
 * Time: 15:00
 * To change this template use File | Settings | File Templates.
 */
abstract class ValidacaoEmissaoBoleto
{

    /** @var $boleto \Financeiro\Entities\Boletreceber */
    protected $boleto;

    public function __construct(\Financeiro\Entities\Boletreceber $boleto)
    {
        $this->boleto = $boleto;
    }

    public abstract function valida();

    public static function factory(\Financeiro\Entities\Boletreceber $boleto)
    {
        try {
            self::validacoesGerais($boleto);
            $codigoCompensacao = $boleto->getReducont()->getAgencia()->getBanco()->getCdbanccnab();
            return self::defineBancoValidacao($codigoCompensacao, $boleto);
        } catch (\Exception $e) {
            throw $e;

        }
    }

    private static function defineBancoValidacao($codigoCompensacao, $boleto)
    {
        if ($codigoCompensacao === EmissaoBoleto::CODIGO_COMPENSACAO_SANTANDER) {
            return new ValidacaoEmissaoBoletoSantander($boleto);
		}
		if ($codigoCompensacao === EmissaoBoleto::CODIGO_COMPENSACAO_ITAU) {
            return new ValidacaoEmissaoBoletoItau($boleto);	
        }	
		if ($codigoCompensacao === EmissaoBoleto::CODIGO_COMPENSACAO_CAIXA) {
            return new ValidacaoEmissaoBoletoCaixa($boleto);	
        }	
		if ($codigoCompensacao === EmissaoBoleto::CODIGO_COMPENSACAO_BRASIL) {
            return new ValidacaoEmissaoBoletoBrasil($boleto);	
        }
        if ($codigoCompensacao === EmissaoBoleto::CODIGO_COMPENSACAO_BANRISUL) {
            return new ValidacaoEmissaoBoletoBanrisul($boleto);
        }
        if ($codigoCompensacao === EmissaoBoleto::CODIGO_COMPENSACAO_BRADESCO) {
            return new ValidacaoEmissaoBoletoBradesco($boleto);

        }else {
            Exception\CodigoCompensacaoInvalido::CnabInvalido();
        }
    }

    private static function validacoesGerais(\Financeiro\Entities\Boletreceber $boleto)
    {
        if ((bool)$boleto->getCdreducont() === false) {
            Exception\ExcecaoBoletoGeral::contaCorrenteNaoInformada();
        }
    }


}
